@@ -1,29 +1,34 @@
-#!/usr/bin/env python
-#Copyright (C) 2009-2010 :
+#!/usr/bin/python
+
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2009-2012:
 #    Gabes Jean, naparuba@gmail.com
 #    Gerhard Lausser, Gerhard.Lausser@consol.de
 #    Gregory Starck, g.starck@gmail.com
 #    Hartmut Goebel, h.goebel@goebel-consult.de
 #
-#This file is part of Shinken.
+# This file is part of Shinken.
 #
-#Shinken is free software: you can redistribute it and/or modify
-#it under the terms of the GNU Affero General Public License as published by
-#the Free Software Foundation, either version 3 of the License, or
-#(at your option) any later version.
+# Shinken is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-#Shinken is distributed in the hope that it will be useful,
-#but WITHOUT ANY WARRANTY; without even the implied warranty of
-#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#GNU Affero General Public License for more details.
+# Shinken is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
 #
-#You should have received a copy of the GNU Affero General Public License
-#along with Shinken.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Affero General Public License
+# along with Shinken.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
 import sys
 import time
 import traceback
+import socket
+import cPickle
 
 from multiprocessing import active_children
 from Queue import Empty
@@ -39,6 +44,13 @@ from shinken.pyro_wrapper import Pyro
 
 from shinken.external_command import ExternalCommand
 
+# Pack of common Pyro exceptions
+#Pyro_exp_pack = (Pyro.errors.ProtocolError, Pyro.errors.URIError, \
+#                    Pyro.errors.CommunicationError, \
+#                    Pyro.errors.DaemonError, Pyro.errors.TimeoutError)
+
+from shinken.pyro_wrapper import Pyro_exp_pack
+
 
 # Our main APP class
 class Broker(BaseSatellite):
@@ -50,9 +62,8 @@ class Broker(BaseSatellite):
         'local_log': PathProp(default='brokerd.log'),
     })
 
-
     def __init__(self, config_file, is_daemon, do_replace, debug, debug_file):
-        
+
         super(Broker, self).__init__('broker', config_file, is_daemon, do_replace, debug, debug_file)
 
         # Our arbiters
@@ -65,76 +76,75 @@ class Broker(BaseSatellite):
         # Modules are load one time
         self.have_modules = False
 
-        # Can have a queue of external_commands give by modules
-        # will be taken by arbiter to process
+        # Can have a queue of external_commands given by modules
+        # will be processed by arbiter
         self.external_commands = []
 
         # All broks to manage
-        self.broks = [] # broks to manage
-        # broks raised this turn and that need to be put in self.broks
+        self.broks = []  # broks to manage
+        # broks raised this turn and that needs to be put in self.broks
         self.broks_internal_raised = []
 
         self.timeout = 1.0
 
-
-    # Schedulers have some queues. We can simplify call by adding
+    # Schedulers have some queues. We can simplify the call by adding
     # elements into the proper queue just by looking at their type
     # Brok -> self.broks
-    # TODO : better tag ID?
+    # TODO: better tag ID?
     # External commands -> self.external_commands
     def add(self, elt):
         cls_type = elt.__class__.my_type
         if cls_type == 'brok':
             # For brok, we TAG brok with our instance_id
-            elt.data['instance_id'] = 0
+            elt.instance_id = 0
             self.broks_internal_raised.append(elt)
             return
         elif cls_type == 'externalcommand':
-            print "Adding in queue an external command", ExternalCommand.__dict__
+            logger.debug("Enqueuing an external command '%s'" % str(ExternalCommand.__dict__))
             self.external_commands.append(elt)
         # Maybe we got a Message from the modules, it's way to ask something
         # like from now a full data from a scheduler for example.
         elif cls_type == 'message':
             # We got a message, great!
-            print elt.__dict__
+            logger.debug(str(elt.__dict__))
             if elt.get_type() == 'NeedData':
                 data = elt.get_data()
-                # Full instance id mean : I got no data for this scheduler
+                # Full instance id means: I got no data for this scheduler
                 # so give me all dumbass!
                 if 'full_instance_id' in data:
                     c_id = data['full_instance_id']
-                    logger.log('A module is asking me to get all initial data from the scheduler %d' % c_id)
-                    # so we just reset the connection adn the running_id, it will just get all new things
+                    source = elt.source
+                    logger.info('The module %s is asking me to get all initial data from the scheduler %d' % (source, c_id))
+                    # so we just reset the connection and the running_id, it will just get all new things
                     try:
                         self.schedulers[c_id]['con'] = None
                         self.schedulers[c_id]['running_id'] = 0
-                    except KeyError: # maybe this instance was not known, forget it
-                        print "WARNING: a module ask me a full_instance_id for an unknown ID!", c_id
-            # Maybe a module say me that it's dead, I must log it's last words...
+                    except KeyError:  # maybe this instance was not known, forget it
+                        logger.warning("the module %s ask me a full_instance_id for an unknown ID (%d)!" % (source, c_id))
+            # Maybe a module tells me that it's dead, I must log it's last words...
             if elt.get_type() == 'ICrash':
                 data = elt.get_data()
-                logger.log('ERROR : the module %s just crash! Please look at the traceback:' % data['name'])
-                logger.log(data['trace'])
-                # The module dead will be look elsewhere and put in restarted.
+                logger.error('the module %s just crash! Please look at the traceback:' % data['name'])
+                logger.error(data['trace'])
+
+                # The module death will be looked for elsewhere and restarted.
 
 
-    # Get teh good tabs for links by the kind. If unknown, return None
+    # Get the good tabs for links by the kind. If unknown, return None
     def get_links_from_type(self, type):
-        t = {'scheduler' : self.schedulers, 'arbiter' : self.arbiters, \
-             'poller' : self.pollers, 'reactionner' : self.reactionners}
-        if type in t :
+        t = {'scheduler': self.schedulers, 'arbiter': self.arbiters, \
+             'poller': self.pollers, 'reactionner': self.reactionners}
+        if type in t:
             return t[type]
         return None
 
-
     # Call by arbiter to get our external commands
     def get_external_commands(self):
         res = self.external_commands
         self.external_commands = []
         return res
 
-
-    # Check if we do not connect to ofthen to this
+    # Check if we do not connect to often to this
     def is_connection_try_too_close(self, elt):
         now = time.time()
         last_connection = elt['last_connection']
@@ -142,14 +152,13 @@ class Broker(BaseSatellite):
             return  True
         return False
 
-
-    # initialise or re-initialise connection with scheduler or
+    # initialize or re-initialize connection with scheduler or
     # arbiter if type == arbiter
     def pynag_con_init(self, id, type='scheduler'):
-            # Get teh good links tab for looping..
+        # Get the good links tab for looping..
         links = self.get_links_from_type(type)
         if links is None:
-            logger.log('DBG: Type unknown for connection! %s' % type)
+            logger.debug('Type unknown for connection! %s' % type)
             return
 
         if type == 'scheduler':
@@ -170,66 +179,77 @@ class Broker(BaseSatellite):
         running_id = links[id]['running_id']
         # DBG: print "Running id before connection", running_id
         uri = links[id]['uri']
-        links[id]['con'] = Pyro.core.getProxyForURI(uri)
 
         try:
-                # intial ping must be quick
+            socket.setdefaulttimeout(3)
+            links[id]['con'] = Pyro.core.getProxyForURI(uri)
+            socket.setdefaulttimeout(None)
+        except Pyro_exp_pack, exp:
+            # But the multiprocessing module is not compatible with it!
+            # so we must disable it immediately after
+            socket.setdefaulttimeout(None)
+            logger.info("Connection problem to the %s %s: %s" % (type, links[id]['name'], str(exp)))
+            links[id]['con'] = None
+            return
+
+
+        try:
+            # initial ping must be quick
             pyro.set_timeout(links[id]['con'], 5)
             links[id]['con'].ping()
             new_run_id = links[id]['con'].get_running_id()
-            # data transfert can be longer
+            # data transfer can be longer
             pyro.set_timeout(links[id]['con'], 120)
 
-            # The schedulers have been restart : it has a new run_id.
+            # The schedulers have been restarted: it has a new run_id.
             # So we clear all verifs, they are obsolete now.
             if new_run_id != running_id:
-                print "[%s] New running id for the %s %s : %s (was %s)" % (self.name, type, links[id]['name'], new_run_id, running_id)
+                logger.debug("[%s] New running id for the %s %s: %s (was %s)" % (self.name, type, links[id]['name'], new_run_id, running_id))
                 links[id]['broks'].clear()
-                # we must ask for a enw full broks if
+                # we must ask for a new full broks if
                 # it's a scheduler
                 if type == 'scheduler':
-                    print "[%s] I ask for a broks generation to the scheduler %s" % (self.name, links[id]['name'])
-                    links[id]['con'].fill_initial_broks()
-            # else:
-            #     print "I do nto ask for brok generation"
+                    logger.debug("[%s] I ask for a broks generation to the scheduler %s" % (self.name, links[id]['name']))
+                    links[id]['con'].fill_initial_broks(self.name)
+            # Ok all is done, we can save this new running id
             links[id]['running_id'] = new_run_id
-        except (Pyro.errors.ProtocolError, Pyro.errors.CommunicationError), exp:
-            logger.log("[%s] Connexion problem to the %s %s : %s" % (self.name, type, links[id]['name'], str(exp)))
+        except Pyro_exp_pack, exp:
+            logger.info("Connection problem to the %s %s: %s" % (type, links[id]['name'], str(exp)))
             links[id]['con'] = None
             return
-        except Pyro.errors.NamingError, exp:
-            logger.log("[%s] the %s '%s' is not initilised : %s" % (self.name, type, links[id]['name'], str(exp)))
-            links[id]['con'] = None
-            return
-        except KeyError , exp:
-            logger.log("[%s] the %s '%s' is not initilised : %s" % (self.name, type, links[id]['name'], str(exp)))
+#        except Pyro.errors.NamingError, exp:
+#            logger.info("[%s] the %s '%s' is not initialized: %s" % (self.name, type, links[id]['name'], str(exp)))
+#            links[id]['con'] = None
+#            return
+        except KeyError, exp:
+            logger.info("the %s '%s' is not initialized: %s" % (type, links[id]['name'], str(exp)))
             links[id]['con'] = None
             traceback.print_stack()
             return
 
-        logger.log("[%s] Connexion OK to the %s %s" % (self.name, type, links[id]['name']))
-
+        logger.info("Connection OK to the %s %s" % (type, links[id]['name']))
 
     # Get a brok. Our role is to put it in the modules
-    # THEY MUST DO NOT CHANGE data of b !!!
+    # DO NOT CHANGE data of b!!!
     # REF: doc/broker-modules.png (4-5)
     def manage_brok(self, b):
+        #logger.info("manage brok")
+
         # Call all modules if they catch the call
         for mod in self.modules_manager.get_internal_instances():
             try:
                 mod.manage_brok(b)
-            except Exception , exp:
-                print exp.__dict__
-                logger.log("[%s] Warning : The mod %s raise an exception: %s, I'm tagging it to restart later" % (self.name, mod.get_name(),str(exp)))
-                logger.log("[%s] Exception type : %s" % (self.name, type(exp)))
-                logger.log("Back trace of this kill: %s" % (traceback.format_exc()))
+            except Exception, exp:
+                logger.debug(str(exp.__dict__))
+                logger.warning("The mod %s raise an exception: %s, I'm tagging it to restart later" % (mod.get_name(), str(exp)))
+                logger.warning("Exception type: %s" % type(exp))
+                logger.warning("Back trace of this kill: %s" % (traceback.format_exc()))
                 self.modules_manager.set_to_restart(mod)
 
-
     # Add broks (a tab) to different queues for
     # internal and external modules
     def add_broks_to_queue(self, broks):
-        # Ok now put in queue brocks for manage by
+        # Ok now put in queue broks to be managed by
         # internal modules
         self.broks.extend(broks)
 
@@ -241,9 +261,9 @@ class Broker(BaseSatellite):
         self.broks_internal_raised = []
 
     # Get 'objects' from external modules
-    # from now nobody use it, but it can be useful
-    # for a moduel like livestatus to raise external
-    # commandsfor example
+    # right now on nobody uses it, but it can be useful
+    # for modules like livestatus to raise external
+    # commands for example
     def get_objects_from_from_queues(self):
         for f in self.modules_manager.get_external_from_queues():
             full_queue = True
@@ -251,16 +271,16 @@ class Broker(BaseSatellite):
                 try:
                     o = f.get(block=False)
                     self.add(o)
-                except Empty :
+                except Empty:
                     full_queue = False
 
     # We get new broks from schedulers
     # REF: doc/broker-modules.png (2)
     def get_new_broks(self, type='scheduler'):
-            # Get teh good links tab for looping..
+        # Get the good links tab for looping..
         links = self.get_links_from_type(type)
         if links is None:
-            logger.log('DBG: Type unknown for connection! %s' % type)
+            logger.debug('Type unknown for connection! %s' % type)
             return
 
         # We check for new check in each schedulers and put
@@ -268,46 +288,51 @@ class Broker(BaseSatellite):
         for sched_id in links:
             try:
                 con = links[sched_id]['con']
-                if con is not None: # None = not initilized
-                    tmp_broks = con.get_broks()
+                if con is not None:  # None = not initialized
+                    t0 = time.time()
+                    tmp_broks = con.get_broks(self.name)
+                    logger.debug("%s Broks get in %s" % (len(tmp_broks), time.time() - t0))
                     for b in tmp_broks.values():
                         b.instance_id = links[sched_id]['instance_id']
 
                     # Ok, we can add theses broks to our queues
                     self.add_broks_to_queue(tmp_broks.values())
 
-                else: # no con? make the connection
+                else:  # no con? make the connection
                     self.pynag_con_init(sched_id, type=type)
-            # Ok, con is not know, so we create it
-            except KeyError , exp:
-                print exp
+            # Ok, con is not known, so we create it
+            except KeyError, exp:
+                logger.debug("Key error for get_broks : %s" % str(exp))
+                try:
+                    logger.debug(''.join(Pyro.util.getPyroTraceback(exp)))
+                except:
+                    pass
                 self.pynag_con_init(sched_id, type=type)
-            except Pyro.errors.ProtocolError , exp:
-                logger.log("[%s] Connexion problem to the %s %s : %s" % (self.name, type, links[sched_id]['name'], str(exp)))
+            except Pyro.errors.ProtocolError, exp:
+                logger.warning("Connection problem to the %s %s: %s" % (type, links[sched_id]['name'], str(exp)))
                 links[sched_id]['con'] = None
             # scheduler must not #be initialized
-            except AttributeError , exp:
-                logger.log("[%s] The %s %s should not be initialized : %s" % (self.name, type, links[sched_id]['name'], str(exp)))
+            except AttributeError, exp:
+                logger.warning("The %s %s should not be initialized: %s" % (type, links[sched_id]['name'], str(exp)))
             # scheduler must not have checks
-            except Pyro.errors.NamingError , exp:
-                logger.log("[%s] The %s %s should not be initialized : %s" % (self.name, type, links[sched_id]['name'], str(exp)))
-            except Pyro.errors.ConnectionClosedError , exp:
-                logger.log("[%s] Connexion problem to the %s %s : %s" % (self.name, type, links[sched_id]['name'], str(exp)))
+            except Pyro.errors.NamingError, exp:
+                logger.warning("The %s %s should not be initialized: %s" % (type, links[sched_id]['name'], str(exp)))
+            except (Pyro.errors.ConnectionClosedError, Pyro.errors.TimeoutError), exp:
+                logger.warning("Connection problem to the %s %s: %s" % (type, links[sched_id]['name'], str(exp)))
                 links[sched_id]['con'] = None
-            #  What the F**k? We do not know what happenned,
+            #  What the F**k? We do not know what happened,
             # so.. bye bye :)
-            except Exception,x:
-                print x.__class__
-                print x.__dict__
-                logger.log(str(x))
-                logger.log(''.join(Pyro.util.getPyroTraceback(x)))
+            except Exception, x:
+                logger.error(str(x))
+                logger.error(''.join(Pyro.util.getPyroTraceback(x)))
                 sys.exit(1)
 
-        
+
     # Helper function for module, will give our broks
     def get_retention_data(self):
         return self.broks
 
+
     # Get back our broks from a retention module
     def restore_retention_data(self, data):
         self.broks.extend(data)
@@ -319,27 +344,39 @@ class Broker(BaseSatellite):
             a.terminate()
             a.join(1)
         super(Broker, self).do_stop()
-        
-        
+
+
     def setup_new_conf(self):
         conf = self.new_conf
         self.new_conf = None
         self.cur_conf = conf
         # Got our name from the globals
-        if 'broker_name' in conf['global']:
-            name = conf['global']['broker_name']
+        g_conf = conf['global']
+        if 'broker_name' in g_conf:
+            name = g_conf['broker_name']
         else:
             name = 'Unnamed broker'
         self.name = name
-        self.log.load_obj(self, name)
+        logger.load_obj(self, name)
 
-        print "[%s] Sending us configuration %s" % (self.name, conf)
+        logger.debug("[%s] Sending us configuration %s" % (self.name, conf))
         # If we've got something in the schedulers, we do not
         # want it anymore
         # self.schedulers.clear()
         for sched_id in conf['schedulers']:
-            # Must look if we already have it to do nto overdie our broks
-            already_got = sched_id in self.schedulers
+            # Must look if we already have it to do not overdie our broks
+            already_got = False
+
+            # We can already got this conf id, but with another address
+            if sched_id in self.schedulers:
+                new_addr = conf['schedulers'][sched_id]['address']
+                old_addr = self.schedulers[sched_id]['address']
+                new_port = conf['schedulers'][sched_id]['port']
+                old_port = self.schedulers[sched_id]['port']
+                # Should got all the same to be ok :)
+                if new_addr == old_addr and new_port == old_port:
+                    already_got = True
+
             if already_got:
                 broks = self.schedulers[sched_id]['broks']
                 running_id = self.schedulers[sched_id]['running_id']
@@ -348,16 +385,21 @@ class Broker(BaseSatellite):
                 running_id = 0
             s = conf['schedulers'][sched_id]
             self.schedulers[sched_id] = s
+
+            # replacing scheduler address and port by those defined in satellitemap
+            if s['name'] in g_conf['satellitemap']:
+                s = dict(s)  # make a copy
+                s.update(g_conf['satellitemap'][s['name']])
             uri = pyro.create_uri(s['address'], s['port'], 'Broks', self.use_ssl)
             self.schedulers[sched_id]['uri'] = uri
+
             self.schedulers[sched_id]['broks'] = broks
             self.schedulers[sched_id]['instance_id'] = s['instance_id']
             self.schedulers[sched_id]['running_id'] = running_id
             self.schedulers[sched_id]['active'] = s['active']
             self.schedulers[sched_id]['last_connection'] = 0
 
-
-        logger.log("[%s] We have our schedulers : %s " % (self.name, self.schedulers))
+        logger.info("We have our schedulers: %s " % self.schedulers)
 
         # Now get arbiter
         for arb_id in conf['arbiters']:
@@ -369,16 +411,22 @@ class Broker(BaseSatellite):
                 broks = {}
             a = conf['arbiters'][arb_id]
             self.arbiters[arb_id] = a
+
+            # replacing arbiter address and port by those defined in satellitemap
+            if a['name'] in g_conf['satellitemap']:
+                a = dict(a)  # make a copy
+                a.update(g_conf['satellitemap'][a['name']])
             uri = pyro.create_uri(a['address'], a['port'], 'Broks', self.use_ssl)
             self.arbiters[arb_id]['uri'] = uri
+
             self.arbiters[arb_id]['broks'] = broks
-            self.arbiters[arb_id]['instance_id'] = 0 # No use so all to 0
+            self.arbiters[arb_id]['instance_id'] = 0  # No use so all to 0
             self.arbiters[arb_id]['running_id'] = 0
             self.arbiters[arb_id]['last_connection'] = 0
 
-            # We do not connect to the arbiter. To connection hang
+            # We do not connect to the arbiter. Connection hangs
 
-        logger.log("[%s] We have our arbiters : %s " % (self.name, self.arbiters))
+        logger.info("We have our arbiters: %s " % self.arbiters)
 
         # Now for pollers
         for pol_id in conf['pollers']:
@@ -392,20 +440,26 @@ class Broker(BaseSatellite):
                 running_id = 0
             p = conf['pollers'][pol_id]
             self.pollers[pol_id] = p
+
+            # replacing poller address and port by those defined in satellitemap
+            if p['name'] in g_conf['satellitemap']:
+                p = dict(p)  # make a copy
+                p.update(g_conf['satellitemap'][p['name']])
             uri = pyro.create_uri(p['address'], p['port'], 'Broks', self.use_ssl)
             self.pollers[pol_id]['uri'] = uri
+
             self.pollers[pol_id]['broks'] = broks
-            self.pollers[pol_id]['instance_id'] = 0 # No use so all to 0
+            self.pollers[pol_id]['instance_id'] = 0  # No use so all to 0
             self.pollers[pol_id]['running_id'] = running_id
             self.pollers[pol_id]['last_connection'] = 0
 
 #                    #And we connect to it
 #                    self.app.pynag_con_init(pol_id, 'poller')
 
-        logger.log("[%s] We have our pollers : %s" % (self.name, self.pollers))
+        logger.info("We have our pollers: %s" % self.pollers)
 
         # Now reactionners
-        for rea_id in conf['reactionners'] :
+        for rea_id in conf['reactionners']:
             # Must look if we already have it
             already_got = rea_id in self.reactionners
             if already_got:
@@ -417,31 +471,45 @@ class Broker(BaseSatellite):
 
             r = conf['reactionners'][rea_id]
             self.reactionners[rea_id] = r
+
+            # replacing reactionner address and port by those defined in satellitemap
+            if r['name'] in g_conf['satellitemap']:
+                r = dict(r)  # make a copy
+                r.update(g_conf['satellitemap'][r['name']])
             uri = pyro.create_uri(r['address'], r['port'], 'Broks', self.use_ssl)
             self.reactionners[rea_id]['uri'] = uri
+
             self.reactionners[rea_id]['broks'] = broks
-            self.reactionners[rea_id]['instance_id'] = 0 # No use so all to 0
+            self.reactionners[rea_id]['instance_id'] = 0  # No use so all to 0
             self.reactionners[rea_id]['running_id'] = running_id
             self.reactionners[rea_id]['last_connection'] = 0
 
 #                    #And we connect to it
 #                    self.app.pynag_con_init(rea_id, 'reactionner')
 
-        logger.log("[%s] We have our reactionners : %s" % (self.name, self.reactionners))
+        logger.info("We have our reactionners: %s" % self.reactionners)
 
         if not self.have_modules:
             self.modules = mods = conf['global']['modules']
             self.have_modules = True
-            logger.log("[%s] We received modules %s " % (self.name,  mods))
+            logger.info("We received modules %s " % mods)
+
+            # Ok now start, or restart them!
+            # Set modules, init them and start external ones
+            self.modules_manager.set_modules(self.modules)
+            self.do_load_modules()
+            self.modules_manager.start_external_instances()
+
+
 
         # Set our giving timezone from arbiter
         use_timezone = conf['global']['use_timezone']
         if use_timezone != 'NOTSET':
-            logger.log("[%s] Setting our timezone to" % (self.name, use_timezone))
+            logger.info("Setting our timezone to %s" % use_timezone)
             os.environ['TZ'] = use_timezone
             time.tzset()
-        
-        # Connexion init with Schedulers
+
+        # Connection init with Schedulers
         for sched_id in self.schedulers:
             self.pynag_con_init(sched_id, type='scheduler')
 
@@ -451,8 +519,7 @@ class Broker(BaseSatellite):
         for rea_id in self.reactionners:
             self.pynag_con_init(rea_id, type='reactionner')
 
-
-    # An arbiter ask us to wait a new conf, so we must clean
+    # An arbiter ask us to wait for a new conf, so we must clean
     # all our mess we did, and close modules too
     def clean_previous_run(self):
         # Clean all lists
@@ -467,10 +534,16 @@ class Broker(BaseSatellite):
         self.have_modules = False
         self.modules_manager.clear_instances()
 
-        
-
     def do_loop_turn(self):
-        print "Begin Loop : manage broks", len(self.broks)
+        logger.debug("Begin Loop: managing old broks (%d)" % len(self.broks))
+
+        # Dump modules Queues size
+        insts = [inst for inst in self.modules_manager.instances if inst.is_external]
+        for inst in insts:
+            try:
+                logger.debug("External Queue len (%s): %s" % (inst.get_name(), inst.to_q.qsize()))
+            except Exception, exp:
+                logger.debug("External Queue len (%s): Exception! %s" % (inst.get_name(), exp))
 
         # Begin to clean modules
         self.check_and_del_zombie_modules()
@@ -481,24 +554,23 @@ class Broker(BaseSatellite):
             # Clean previous run from useless objects
             # and close modules
             self.clean_previous_run()
-            
+
             self.wait_for_initial_conf()
-            # we may have been interrupted or so; then 
+            # we may have been interrupted or so; then
             # just return from this loop turn
-            if not self.new_conf:  
+            if not self.new_conf:
                 return
             self.setup_new_conf()
 
-        # Now we check if arbiter speek to us in the pyro_daemon.
+        # Now we check if arbiter speak to us in the pyro_daemon.
         # If so, we listen for it
-        # When it push us conf, we reinit connections
+        # When it pushes conf to us, we reinit connections
         self.watch_for_new_conf(0.0)
         if self.new_conf:
             self.setup_new_conf()
 
-
         # Maybe the last loop we raised some broks internally
-        # we should interger them in broks
+        # we should integrate them in broks
         self.interger_internal_broks()
 
         # And from schedulers
@@ -509,16 +581,22 @@ class Broker(BaseSatellite):
 
         # Sort the brok list by id
         self.broks.sort(sort_by_ids)
-        
+
         # and for external queues
         # REF: doc/broker-modules.png (3)
         # We put to external queues broks that was not already send
+        t0 = time.time()
+        # We are sending broks as a big list, more efficient than one by one
         queues = self.modules_manager.get_external_to_queues()
-        for b in (b for b in self.broks if getattr(b, 'need_send_to_ext', True)):
-            # if b.type != 'log':
-            for q in queues:
-                q.put(b)
-                b.need_send_to_ext = False
+        to_send = [b for b in self.broks if getattr(b, 'need_send_to_ext', True)]
+
+        for q in queues:
+            q.put(to_send)
+
+        # No more need to send them
+        for b in to_send:
+            b.need_send_to_ext = False
+        logger.debug("Time to send %s broks (%d secs)" % (len(to_send), time.time() - t0))
 
         # We must had new broks at the end of the list, so we reverse the list
         self.broks.reverse()
@@ -534,6 +612,8 @@ class Broker(BaseSatellite):
             b = self.broks.pop()
             # Ok, we can get the brok, and doing something with it
             # REF: doc/broker-modules.png (4-5)
+            # We un serialize the brok before consume it
+            b.prepare()
             self.manage_brok(b)
 
             nb_broks = len(self.broks)
@@ -541,18 +621,18 @@ class Broker(BaseSatellite):
             # Ok we manage brok, but we still want to listen to arbiter
             self.watch_for_new_conf(0.0)
 
-            # if we got new broks here from arbiter, we should breack the loop
+            # if we got new broks here from arbiter, we should break the loop
             # because such broks will not be managed by the
             # external modules before this loop (we pop them!)
             if len(self.broks) != nb_broks:
                 break
 
-        # Maybe external modules raised 'objets'
+        # Maybe external modules raised 'objects'
         # we should get them
         self.get_objects_from_from_queues()
 
         # Maybe we do not have something to do, so we wait a little
-        #TODO : redone the diff management....
+        # TODO: redone the diff management....
         if len(self.broks) == 0:
             while self.timeout > 0:
                 begin = time.time()
@@ -560,23 +640,28 @@ class Broker(BaseSatellite):
                 end = time.time()
                 self.timeout = self.timeout - (end - begin)
             self.timeout = 1.0
-            # print "get enw broks watch new conf 1 : end", len(self.broks)
 
+            # print "get new broks watch new conf 1: end", len(self.broks)
+
+        # Say to modules it's a new tick :)
+        self.hook_point('tick')
 
     #  Main function, will loop forever
     def main(self):
         try:
             self.load_config_file()
-        
-            for line in self.get_header():
-                self.log.log(line)
 
-            logger.log("[Broker] Using working directory : %s" % os.path.abspath(self.workdir))
-        
+            for line in self.get_header():
+                logger.info(line)
+
+            logger.info("[Broker] Using working directory: %s" % os.path.abspath(self.workdir))
+
+            # Look if we are enabled or not. If ok, start the daemon mode
+            self.look_for_early_exit()
             self.do_daemon_init_and_start()
 
             self.uri2 = self.pyro_daemon.register(self.interface, "ForArbiter")
-            print "The Arbiter uri it at", self.uri2
+            logger.debug("The Arbiter uri it at %s" % self.uri2)
 
             #  We wait for initial conf
             self.wait_for_initial_conf()
@@ -585,10 +670,13 @@ class Broker(BaseSatellite):
 
             self.setup_new_conf()
 
+            # We already init modules during the new conf thing
             # Set modules, init them and start external ones
-            self.modules_manager.set_modules(self.modules)
-            self.do_load_modules()
-            self.modules_manager.start_external_instances()
+            #self.modules_manager.set_modules(self.modules)
+            #self.do_load_modules()
+            #self.modules_manager.start_external_instances()
+
+
 
             # Do the modules part, we have our modules in self.modules
             # REF: doc/broker-modules.png (1)
@@ -598,8 +686,7 @@ class Broker(BaseSatellite):
             self.do_mainloop()
 
         except Exception, exp:
-            logger.log("CRITICAL ERROR : I got an non recovarable error. I must exit")
-            logger.log("You can log a bug ticket at https://sourceforge.net/apps/trac/shinken/newticket for geting help")
-            logger.log("Back trace of it: %s" % (traceback.format_exc()))
+            logger.critical("I got an unrecoverable error. I have to exit")
+            logger.critical("You can log a bug ticket at https://github.com/naparuba/shinken/issues/new to get help")
+            logger.critical("Back trace of it: %s" % (traceback.format_exc()))
             raise
-
